@@ -6,8 +6,6 @@ function QueueListModel(parent) {
     var self = this;
     self.parent = parent;
     self.dragging = false;
-    self.rawCatList = [];
-    self.rawScriptList = [];
 
     // Because SABNZB returns the name
     // But when you want to set Priority you need the number..
@@ -34,12 +32,13 @@ function QueueListModel(parent) {
     // External var's
     self.queueItems = ko.observableArray([]);
     self.totalItems = ko.observable(0);
-    self.isMultiEditing = ko.observable(false);
+    self.isMultiEditing = ko.observable(false).extend({ persist: 'queueIsMultiEditing' });
     self.isLoading = ko.observable(false).extend({ rateLimit: 100 });
     self.multiEditItems = ko.observableArray([]);
     self.categoriesList = ko.observableArray([]);
     self.scriptsList = ko.observableArray([]);
-    self.searchTerm = ko.observable('').extend({ rateLimit: { timeout: 200, method: "notifyWhenChangesStop" } });
+    self.scriptsListLoaded = ko.observable(false);
+    self.searchTerm = ko.observable('').extend({ rateLimit: { timeout: 400, method: "notifyWhenChangesStop" } });
     self.paginationLimit = ko.observable(20).extend({ persist: 'queuePaginationLimit' });
     self.pagination = new paginationModel(self);
 
@@ -66,31 +65,6 @@ function QueueListModel(parent) {
             return i.id;
         });
 
-        // Did the category-list change?
-        // Otherwise KO will send updates to all <select> for every refresh()
-        if(self.rawCatList != data.categories.toString()) {
-            // Reformat categories
-            self.categoriesList($.map(data.categories, function(cat) {
-                // Default?
-                if(cat == '*') return { catValue: '*', catText: glitterTranslate.defaultText };
-                return { catValue: cat, catText: cat };
-            }))
-            // Update
-            self.rawCatList = data.categories.toString();
-        }
-
-        // Did the script-list change?
-        if(self.rawScriptList != data.scripts.toString()) {
-            // Reformat script-list
-            self.scriptsList($.map(data.scripts, function(script) {
-                // Default?
-                if(script == 'None') return glitterTranslate.noneText;
-                return script;
-            }))
-            // Update
-            self.rawScriptList = data.scripts.toString();
-        }
-
         // Set limit
         self.totalItems(data.noofslots);
 
@@ -194,11 +168,13 @@ function QueueListModel(parent) {
 
     // Searching in queue (rate-limited in decleration)
     self.searchTerm.subscribe(function() {
-        // Refresh now
-        self.parent.refresh();
         // Go back to page 1
         if(self.pagination.currentPage() != 1) {
+            // This forces a refresh
             self.pagination.moveToPage(1);
+        } else {
+            // Refresh now
+            self.parent.refresh();
         }
     })
 
@@ -208,7 +184,6 @@ function QueueListModel(parent) {
         if(event.type == 'mousedown' || (event.keyCode && event.keyCode == 27)) {
             self.isLoading(true)
             self.searchTerm('');
-            self.parent.refresh()
         }
         // Was it click and the field is empty? Then we focus on the field
         if(event.type == 'mousedown' && self.searchTerm() == '') {
@@ -226,6 +201,10 @@ function QueueListModel(parent) {
         // What action?
         var sort, dir;
         switch($(event.currentTarget).data('action')) {
+            case 'sortRemainingAsc':
+                sort = 'remaining';
+                dir = 'asc';
+                break;
             case 'sortAgeAsc':
                 sort = 'avg_age';
                 dir = 'desc';
@@ -364,54 +343,61 @@ function QueueListModel(parent) {
         var newStatus = $('.multioperations-selector input[name="multiedit-status"]:checked').val()
 
         // List all the ID's
-        var  strIDs = '';
+        var strIDs = '';
         $.each(self.multiEditItems(), function(index) {
             strIDs = strIDs + this.id + ',';
         })
 
+        // All non-category updates need to only happen after a category update
+        function nonCatUpdates() {
+            if(newScript != '') {
+                callAPI({
+                    mode: 'change_script',
+                    value: strIDs,
+                    value2: newScript
+                })
+            }
+            if(newPrior != '') {
+                callAPI({
+                    mode: 'queue',
+                    name: 'priority',
+                    value: strIDs,
+                    value2: newPrior
+                })
+            }
+            if(newProc != '') {
+                callAPI({
+                    mode: 'change_opts',
+                    value: strIDs,
+                    value2: newProc
+                })
+            }
+            if(newStatus) {
+                callAPI({
+                    mode: 'queue',
+                    name: newStatus,
+                    value: strIDs
+                })
+            }
+
+            // Wat a little and do the refresh
+            // Only if anything changed!
+            if(newStatus || newProc != '' || newPrior != '' || newScript != '' || newCat != '') {
+                setTimeout(parent.refresh, 100)
+            }
+        }
+
         // What is changed?
         if(newCat != '') {
             callAPI({
                 mode: 'change_cat',
                 value: strIDs,
                 value2: newCat
-            })
-        }
-        if(newScript != '') {
-            callAPI({
-                mode: 'change_script',
-                value: strIDs,
-                value2: newScript
-            })
-        }
-        if(newPrior != '') {
-            callAPI({
-                mode: 'queue',
-                name: 'priority',
-                value: strIDs,
-                value2: newPrior
-            })
-        }
-        if(newProc != '') {
-            callAPI({
-                mode: 'change_opts',
-                value: strIDs,
-                value2: newProc
-            })
-        }
-        if(newStatus) {
-            callAPI({
-                mode: 'queue',
-                name: newStatus,
-                value: strIDs
-            })
+            }).then(nonCatUpdates)
+        } else {
+            nonCatUpdates()
         }
 
-        // Wat a little and do the refresh
-        // Only if anything changed!
-        if(newStatus || newProc != '' || newPrior != '' || newScript != '' || newCat != '') {
-            setTimeout(parent.refresh, 100)
-        }
     }
 
     // Selete all selected
@@ -444,7 +430,7 @@ function QueueListModel(parent) {
                     // Empty it
                     self.multiEditItems.removeAll();
                     // Hide notification
-                    hideNotification(true)
+                    hideNotification()
                 }
             })
         }
@@ -531,11 +517,10 @@ function QueueModel(parent, data) {
         return self.name()
     })
     self.missingText = ko.pureComputed(function() {
-        // Check for missing data, the value is arbitrary! (1%)
-        if(self.missingMB()/self.totalMB() > 0.01) {
+        // Check for missing data, can show 0 if article-size is smaller than 500K, but we accept that
+        if(self.missingMB()) {
             return self.missingMB().toFixed(0) + ' MB ' + glitterTranslate.misingArt
         }
-        return;
     })
     self.statusText = ko.computed(function() {
         // Checking
@@ -694,8 +679,6 @@ function QueueModel(parent, data) {
         })
     }
     self.changeScript = function(item) {
-        // Not on empty handlers
-        if(!item.script() || parent.scriptsList().length <= 1) return;
         callAPI({
             mode: 'change_script',
             value: item.id,
@@ -745,7 +728,7 @@ function QueueModel(parent, data) {
                 parent.multiEditItems.remove(function(inList) { return inList.id == itemToDelete.id; })
                 self.parent.parent.refresh();
                 // Hide notifcation
-                hideNotification(true)
+                hideNotification()
             });
         }
     };
