@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -196,6 +196,7 @@ CMDLINE = " ".join(['"%s"' % arg for arg in sys.argv])
 __INITIALIZED__ = False
 __SHUTTING_DOWN__ = False
 
+
 ##############################################################################
 # Signal Handler
 ##############################################################################
@@ -512,6 +513,9 @@ def check_all_tasks():
     # Check (and terminate) idle jobs
     sabnzbd.NzbQueue.stop_idle_jobs()
 
+    # Check that the queue is sorted correctly
+    sabnzbd.NzbQueue.update_sort_order()
+
     return True
 
 
