@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -48,7 +48,7 @@ class ArticleCache:
         # On 32 bit we only allow the user to set 1GB
         # For 64 bit we allow up to 4GB, in case somebody wants that
         self.__cache_upper_limit = GIGI
-        if sabnzbd.DARWIN or sabnzbd.WIN64 or (struct.calcsize("P") * 8) == 64:
+        if sabnzbd.MACOS or sabnzbd.WIN64 or (struct.calcsize("P") * 8) == 64:
             self.__cache_upper_limit = 4 * GIGI
 
     def cache_info(self):
@@ -85,9 +85,8 @@ class ArticleCache:
     def save_article(self, article: Article, data: bytes):
         """Save article in cache, either memory or disk"""
         nzo = article.nzf.nzo
-        if nzo.is_gone():
-            # Do not discard this article because the
-            # file might still be processed at this moment!!
+        # Skip if already post-processing or fully finished
+        if nzo.pp_or_finished:
             return
 
         # Register article for bookkeeping in case the job is deleted
@@ -129,7 +128,9 @@ class ArticleCache:
                 logging.debug("Failed to load %s from cache, probably already deleted", article)
                 return data
         elif article.art_id:
-            data = sabnzbd.load_data(article.art_id, nzo.admin_path, remove=True, do_pickle=False, silent=True)
+            data = sabnzbd.filesystem.load_data(
+                article.art_id, nzo.admin_path, remove=True, do_pickle=False, silent=True
+            )
         nzo.remove_saved_article(article)
         return data
 
@@ -156,15 +157,12 @@ class ArticleCache:
                     # Could fail if already deleted by flush_articles or load_data
                     logging.debug("Failed to flush %s from cache, probably already deleted or written to disk", article)
             elif article.art_id:
-                sabnzbd.remove_data(article.art_id, article.nzf.nzo.admin_path)
+                sabnzbd.filesystem.remove_data(article.art_id, article.nzf.nzo.admin_path)
 
     @staticmethod
     def __flush_article_to_disk(article: Article, data):
-        nzo = article.nzf.nzo
-        if nzo.is_gone():
-            # Don't store deleted jobs
-            return
-
         # Save data, but don't complain when destination folder is missing
         # because this flush may come after completion of the NZO.
-        sabnzbd.save_data(data, article.get_art_id(), nzo.admin_path, do_pickle=False, silent=True)
+        sabnzbd.filesystem.save_data(
+            data, article.get_art_id(), article.nzf.nzo.admin_path, do_pickle=False, silent=True
+        )
