@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@ import os
 import re
 import shutil
 import threading
+import time
 import uuid
 import io
 import zipfile
@@ -393,7 +394,6 @@ class ConfigServer:
     """Class defining a single server"""
 
     def __init__(self, name, values):
-
         self.__name = clean_section_name(name)
         name = "servers," + self.__name
 
@@ -649,12 +649,15 @@ class ConfigRSS:
         """Remove from database"""
         delete_from_database("rss", self.__name)
 
-    def rename(self, new_name: str):
+    def rename(self, new_name: str) -> str:
         """Update the name and the saved entries"""
+        # Sanitize the name before using it
+        new_name = clean_section_name(new_name)
         delete_from_database("rss", self.__name)
         sabnzbd.RSSReader.rename(self.__name, new_name)
         self.__name = new_name
         add_to_database("rss", self.__name, self)
+        return self.__name
 
 
 # Add typing to the options database-dict
@@ -709,7 +712,9 @@ def get_dconfig(section, keyword, nested=False):
             sect = CFG_DATABASE[section]
         except KeyError:
             return False, {}
-        if section in ("servers", "categories", "rss"):
+        if section == "categories":
+            data[section] = get_ordered_categories()
+        elif section in ("servers", "rss"):
             data[section] = []
             for keyword in sect.keys():
                 res, conf = get_dconfig(section, keyword, True)
@@ -917,20 +922,33 @@ def save_config(force=False):
     return res
 
 
-def create_config_backup():
-    """Put config data in a zip file"""
-    adminpath = sabnzbd.cfg.admin_dir.get_path()
-    logging.debug("Backing up %s + %s", adminpath, CFG_OBJ.filename)
-    with io.BytesIO() as zip_buffer:
-        with zipfile.ZipFile(zip_buffer, "a", zipfile.ZIP_DEFLATED, False) as zip_ref:
-            for filename in CONFIG_BACKUP_FILES:
-                full_path = os.path.join(adminpath, filename)
-                if os.path.isfile(full_path):
-                    with open(full_path, "rb") as data:
-                        zip_ref.writestr(filename, data.read())
-            with open(CFG_OBJ.filename, "rb") as data:
-                zip_ref.writestr(DEF_INI_FILE, data.read())
-        return zip_buffer.getvalue()
+def create_config_backup() -> Union[str, bool]:
+    """Put config data in a zip file, returns path on success"""
+    admin_path = sabnzbd.cfg.admin_dir.get_path()
+    output_filename = "sabnzbd_backup_%s_%s.zip" % (sabnzbd.__version__, time.strftime("%Y.%m.%d_%H.%M.%S"))
+
+    # Check if there is a backup folder set, use complete otherwise
+    if sabnzbd.cfg.backup_dir():
+        backup_dir = sabnzbd.cfg.backup_dir.get_path()
+    else:
+        backup_dir = sabnzbd.cfg.complete_dir.get_path()
+    complete_path = os.path.join(backup_dir, output_filename)
+    logging.debug("Backing up %s + %s in %s", admin_path, CFG_OBJ.filename, complete_path)
+
+    try:
+        with open(complete_path, "wb") as zip_buffer:
+            with zipfile.ZipFile(zip_buffer, "a", zipfile.ZIP_DEFLATED, False) as zip_ref:
+                for filename in CONFIG_BACKUP_FILES:
+                    full_path = os.path.join(admin_path, filename)
+                    if os.path.isfile(full_path):
+                        with open(full_path, "rb") as data:
+                            zip_ref.writestr(filename, data.read())
+                with open(CFG_OBJ.filename, "rb") as data:
+                    zip_ref.writestr(DEF_INI_FILE, data.read())
+        return clip_path(complete_path)
+    except:
+        logging.info("Failed to create backup: ", exc_info=True)
+        return False
 
 
 def validate_config_backup(config_backup_data: bytes) -> bool:
@@ -945,7 +963,7 @@ def validate_config_backup(config_backup_data: bytes) -> bool:
         return False
 
 
-def restore_config_backup(config_backup_data: bytes) -> bool:
+def restore_config_backup(config_backup_data: bytes):
     """Restore configuration files from zip file"""
     try:
         with io.BytesIO(config_backup_data) as backup_ref:
