@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -398,11 +398,11 @@ class DirectUnpacker(threading.Thread):
         # Generate command
         rarfile_path = os.path.join(self.nzo.download_path, self.rarfile_nzf.filename)
         if sabnzbd.WIN32:
-            # For Unrar to support long-path, we need to circumvent Python's list2cmdline
+            # On Windows, UnRar uses a custom argument parser
             # See: https://github.com/sabnzbd/sabnzbd/issues/1043
             # The -scf forces the output to be UTF8
             command = [
-                "%s" % sabnzbd.newsunpack.RAR_COMMAND,
+                sabnzbd.newsunpack.RAR_COMMAND,
                 action,
                 "-vp",
                 "-idp",
@@ -414,17 +414,17 @@ class DirectUnpacker(threading.Thread):
                 "%s\\" % long_path(extraction_path),
             ]
         else:
-            # Don't use "-ai" (not needed for non-Windows)
             # The -scf forces the output to be UTF8
             command = [
-                "%s" % sabnzbd.newsunpack.RAR_COMMAND,
+                sabnzbd.newsunpack.RAR_COMMAND,
                 action,
                 "-vp",
                 "-idp",
                 "-scf",
                 "-o+",
+                "-ai",
                 password_command,
-                "%s" % rarfile_path,
+                rarfile_path,
                 "%s/" % extraction_path,
             ]
 
@@ -435,7 +435,7 @@ class DirectUnpacker(threading.Thread):
         self.cur_volume = 1
 
         # Need to disable buffer to have direct feedback
-        self.active_instance = build_and_run_command(command, flatten_command=True, bufsize=0)
+        self.active_instance = build_and_run_command(command, windows_unrar_command=True, bufsize=0)
 
         # Add to runners
         ACTIVE_UNPACKERS.append(self)
@@ -506,7 +506,7 @@ class DirectUnpacker(threading.Thread):
             # Reset settings
             self.reset_active()
 
-    def get_formatted_stats(self, include_time_left: bool = False):
+    def get_formatted_stats(self, include_time_left: bool = False) -> str:
         """Get percentage or number of rar's done"""
         if self.cur_setname and self.cur_setname in self.total_volumes:
             # This won't work on obfuscated posts
@@ -517,7 +517,7 @@ class DirectUnpacker(threading.Thread):
                         (self.cur_volume / self.total_volumes[self.cur_setname]) * 100, time_used=self.unpack_time
                     )
                 return formatted_stats
-        return self.cur_volume
+        return str(self.cur_volume)
 
 
 def analyze_rar_filename(filename):
