@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -35,7 +35,7 @@ from sabnzbd.decorators import synchronized, NzbQueueLocker, DOWNLOADER_CV
 from sabnzbd.newswrapper import NewsWrapper, NNTPPermanentError
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
-from sabnzbd.misc import from_units, nntp_to_msg, get_server_addrinfo, helpful_warning
+from sabnzbd.misc import from_units, nntp_to_msg, get_server_addrinfo, helpful_warning, int_conv
 from sabnzbd.utils.happyeyeballs import happyeyeballs
 
 
@@ -53,6 +53,8 @@ _PENALTY_VERYSHORT = 0.1  # Error 400 without cause clues
 _SERVER_CHECK_DELAY = 0.5
 # Wait this many seconds between updates of the BPSMeter
 _BPSMETER_UPDATE_DELAY = 0.05
+# How many articles should be prefetched when checking the next articles?
+_ARTICLE_PREFETCH = 20
 
 TIMER_LOCK = RLock()
 
@@ -114,7 +116,6 @@ class Server:
         optional=False,
         retention=0,
     ):
-
         self.id: str = server_id
         self.newid: Optional[str] = None
         self.restart: bool = False
@@ -151,12 +152,14 @@ class Server:
         self.have_stat: bool = True  # Assume server has "STAT", until proven otherwise
         self.article_queue: List[sabnzbd.nzbstuff.Article] = []
 
-        # Initialize threads
-        for i in range(threads):
-            self.idle_threads.append(NewsWrapper(self, i + 1))
+        # Skip during server testing
+        if threads:
+            # Initialize threads
+            for i in range(threads):
+                self.idle_threads.append(NewsWrapper(self, i + 1))
 
-        # Tell the BPSMeter about this server
-        sabnzbd.BPSMeter.init_server_stats(self.id)
+            # Tell the BPSMeter about this server
+            sabnzbd.BPSMeter.init_server_stats(self.id)
 
     @property
     def hostip(self) -> str:
@@ -176,27 +179,25 @@ class Server:
         # Determine IP
         ip = self.host
         if self.info:
-            if cfg.load_balancing() == 0 or len(self.info) == 1:
+            # Check this first so we can fall back to default method if it returns None.
+            if len(self.info) > 1 and cfg.load_balancing() == 2:
+                # RFC6555 / Happy Eyeballs:
+                ip = happyeyeballs(self.host, port=self.port)
+                if not ip:
+                    # nothing returned, so there was a connection problem
+                    logging.debug("%s: No successful IP connection was possible using happyeyeballs", self.host)
+            if not ip or cfg.load_balancing() == 0 or len(self.info) == 1:
                 # Just return the first one, so all next threads use the same IP
                 ip = self.info[0][4][0]
-                logging.debug("%s: Connecting to address %s", self.host, ip)
             elif cfg.load_balancing() == 1:
                 # Return a random entry from the possible IPs
                 rnd = random.randint(0, len(self.info) - 1)
                 ip = self.info[rnd][4][0]
-                logging.debug("%s: Connecting to address %s", self.host, ip)
-            elif cfg.load_balancing() == 2:
-                # RFC6555 / Happy Eyeballs:
-                ip = happyeyeballs(self.host, port=self.port)
-                if ip:
-                    logging.debug("%s: Connecting to address %s", self.host, ip)
-                else:
-                    # nothing returned, so there was a connection problem
-                    logging.debug("%s: No successful IP connection was possible", self.host)
+        logging.debug("%s: Connecting to address %s", self.host, ip)
         return ip
 
     def deactivate(self):
-        """Deactive server and reset queued articles"""
+        """Deactivate server and reset queued articles"""
         self.active = False
         self.reset_article_queue()
 
@@ -228,6 +229,7 @@ class Server:
         self.info = get_server_addrinfo(self.host, self.port)
         if not self.info:
             self.bad_cons += self.threads
+            self.info = False
         else:
             self.bad_cons = 0
         self.request = False
@@ -264,7 +266,7 @@ class Downloader(Thread):
         # Used for scheduled pausing
         self.paused: bool = paused
 
-        # Used for reducing speed
+        # Used for reducing speed, should always be int and not float
         self.bandwidth_limit: int = 0
         self.bandwidth_perc: int = 0
         cfg.bandwidth_perc.callback(self.speed_set)
@@ -394,7 +396,7 @@ class Downloader(Thread):
             sabnzbd.notifier.send_notification("SABnzbd", T("Paused"), "pause_resume")
             if cfg.preserve_paused_state():
                 cfg.start_paused.set(True)
-            if self.is_paused():
+            if self.no_active_jobs():
                 sabnzbd.BPSMeter.reset()
             if cfg.autodisconnect():
                 self.disconnect()
@@ -415,40 +417,34 @@ class Downloader(Thread):
 
     def limit_speed(self, value: Union[str, int]):
         """Set the actual download speed in Bytes/sec
-        When 'value' ends with a '%' sign or is within 1-100, it is interpreted as a pecentage of the maximum bandwidth
+        When 'value' ends with a '%' sign or is within 1-100, it is interpreted as a percentage of the maximum bandwidth
         When no '%' is found, it is interpreted as an absolute speed (including KMGT notation).
         """
         if value:
             mx = cfg.bandwidth_max.get_int()
             if "%" in str(value) or (0 < from_units(value) < 101):
                 limit = value.strip(" %")
-                self.bandwidth_perc = from_units(limit)
+                self.bandwidth_perc = int_conv(limit)
                 if mx:
-                    self.bandwidth_limit = mx * self.bandwidth_perc / 100
+                    self.bandwidth_limit = int(mx * self.bandwidth_perc / 100)
                 else:
                     helpful_warning(T("You must set a maximum bandwidth before you can set a bandwidth limit"))
             else:
-                self.bandwidth_limit = from_units(value)
+                self.bandwidth_limit = int(from_units(value))
                 if mx:
-                    self.bandwidth_perc = self.bandwidth_limit / mx * 100
+                    self.bandwidth_perc = int(self.bandwidth_limit / mx * 100)
                 else:
                     self.bandwidth_perc = 100
         else:
             self.speed_set()
         logging.info("Speed limit set to %s B/s", self.bandwidth_limit)
 
-    def get_limit(self):
-        return self.bandwidth_perc
-
-    def get_limit_abs(self):
-        return self.bandwidth_limit
-
     def speed_set(self):
-        limit = cfg.bandwidth_max.get_int()
         perc = cfg.bandwidth_perc()
+        limit = cfg.bandwidth_max.get_int()
         if limit and perc:
-            self.bandwidth_perc = perc
-            self.bandwidth_limit = limit * perc / 100
+            self.bandwidth_perc = int(perc)
+            self.bandwidth_limit = int(limit * perc / 100)
         else:
             self.bandwidth_perc = 0
             self.bandwidth_limit = 0
@@ -457,23 +453,19 @@ class Downloader(Thread):
         self.sleep_time = cfg.downloader_sleep_time() * 0.0001
         logging.debug("Sleep time: %f seconds", self.sleep_time)
 
-    def is_paused(self):
-        if not self.paused:
-            return False
-        else:
-            if sabnzbd.NzbQueue.has_forced_items():
-                return False
-            else:
-                return True
+    def no_active_jobs(self) -> bool:
+        """Is the queue paused or is it paused but are there still forced items?"""
+        return self.paused and not sabnzbd.NzbQueue.has_forced_jobs()
 
     def highest_server(self, me: Server):
         """Return True when this server has the highest priority of the active ones
-        0 is the highest priority
+        0 is the highest priority, servers are sorted by priority.
         """
         for server in self.servers:
-            if server is not me and server.active and server.priority < me.priority:
+            if server.priority == me.priority:
+                return True
+            if server.active:
                 return False
-        return True
 
     def maybe_block_server(self, server: Server):
         # Was it resolving problem?
@@ -510,7 +502,7 @@ class Downloader(Thread):
             # Make sure server address resolution is refreshed
             server.info = None
 
-    def decode(self, article, raw_data: Optional[List[bytes]]):
+    def decode(self, article, raw_data: Optional[List[bytes]] = None, data_size: Optional[int] = None):
         """Decode article and check the status of
         the decoder and the assembler
         """
@@ -522,11 +514,10 @@ class Downloader(Thread):
             if not article.search_new_server():
                 sabnzbd.NzbQueue.register_article(article, success=False)
                 article.nzf.nzo.increase_bad_articles_counter("missing_articles")
-                article.nzf.has_bad_articles = True
             return
 
         # Send to decoder-queue
-        sabnzbd.Decoder.process(article, raw_data)
+        sabnzbd.Decoder.process(article, raw_data, data_size)
 
         # See if we need to delay because the queues are full
         logged_counter = 0
@@ -620,7 +611,7 @@ class Downloader(Thread):
 
                 if (
                     not server.idle_threads
-                    or self.is_paused()
+                    or self.no_active_jobs()
                     or self.shutdown
                     or self.paused_for_postproc
                     or not server.active
@@ -646,16 +637,14 @@ class Downloader(Thread):
                         article = server.article_queue.pop(0)
                     else:
                         # Pre-fetch new articles
-                        server.article_queue = sabnzbd.NzbQueue.get_articles(
-                            server, self.servers, max(1, server.threads // 4)
-                        )
+                        server.article_queue = sabnzbd.NzbQueue.get_articles(server, self.servers, _ARTICLE_PREFETCH)
                         if server.article_queue:
                             article = server.article_queue.pop(0)
                             # Mark expired articles as tried on this server
                             if server.retention and article.nzf.nzo.avg_stamp < now - server.retention:
-                                self.decode(article, None)
+                                self.decode(article)
                                 while server.article_queue:
-                                    self.decode(server.article_queue.pop(), None)
+                                    self.decode(server.article_queue.pop())
                                 # Move to the next server, allowing the next server to already start
                                 # fetching the articles that were too old for this server
                                 break
@@ -744,7 +733,7 @@ class Downloader(Thread):
 
                 with DOWNLOADER_CV:
                     while (
-                        (sabnzbd.NzbQueue.is_empty() or self.is_paused() or self.paused_for_postproc)
+                        (sabnzbd.NzbQueue.is_empty() or self.no_active_jobs() or self.paused_for_postproc)
                         and not self.shutdown
                         and not self.force_disconnect
                         and not self.server_restarts
@@ -764,131 +753,24 @@ class Downloader(Thread):
                 server = nw.server
 
                 try:
-                    bytes_received, done, skip = nw.recv_chunk()
-                except:
-                    bytes_received, done, skip = (0, False, False)
-
-                if skip:
+                    bytes_received, done = nw.recv_chunk()
+                except ssl.SSLWantReadError:
                     continue
-
-                if bytes_received < 1:
+                except:
                     self.__reset_nw(nw, "server closed connection", wait=False)
                     continue
 
-                else:
-                    try:
-                        article.nzf.nzo.update_download_stats(BPSMeter.bps, server.id, bytes_received)
-                    except AttributeError:
-                        # In case nzf has disappeared because the file was deleted before the update could happen
-                        pass
-
-                    BPSMeter.update(server.id, bytes_received)
-
-                    if self.bandwidth_limit:
-                        if BPSMeter.bps + BPSMeter.sum_cached_amount > self.bandwidth_limit:
-                            BPSMeter.update()
-                            while BPSMeter.bps > self.bandwidth_limit:
-                                time.sleep(0.01)
-                                BPSMeter.update()
+                BPSMeter.update(server.id, bytes_received)
+                if self.bandwidth_limit and BPSMeter.bps + BPSMeter.sum_cached_amount > self.bandwidth_limit:
+                    BPSMeter.update()
+                    while BPSMeter.bps > self.bandwidth_limit:
+                        time.sleep(0.01)
+                        BPSMeter.update()
 
                 if nw.status_code != 222 and not done:
                     if not nw.connected or nw.status_code == 480:
-                        try:
-                            nw.finish_connect(nw.status_code)
-                            if sabnzbd.LOG_ALL:
-                                logging.debug(
-                                    "%s@%s last message -> %s", nw.thrdnum, nw.server.host, nntp_to_msg(nw.data)
-                                )
-                            nw.clear_data()
-                        except NNTPPermanentError as error:
-                            # Handle login problems
-                            block = False
-                            penalty = 0
-                            display_msg = " [%s]" % error.msg
-                            logging.debug("Server login problem: %s", error.msg)
-                            if error.code in (502, 400, 481, 482) and clues_too_many(error.msg):
-                                # Too many connections: remove this thread and reduce thread-setting for server
-                                # Plan to go back to the full number after a penalty timeout
-                                if server.active:
-                                    errormsg = T("Too many connections to server %s") % display_msg
-                                    if server.errormsg != errormsg:
-                                        server.errormsg = errormsg
-                                        logging.warning(T("Too many connections to server %s"), server.host)
-                                    # Don't count this for the tries (max_art_tries) on this server
-                                    self.__reset_nw(nw, send_quit=True)
-                                    self.plan_server(server, _PENALTY_TOOMANY)
-                                    server.threads -= 1
-                            elif error.code in (502, 481, 482) and clues_too_many_ip(error.msg):
-                                # Account sharing?
-                                if server.active:
-                                    errormsg = T("Probable account sharing") + display_msg
-                                    if server.errormsg != errormsg:
-                                        server.errormsg = errormsg
-                                        name = " (%s)" % server.host
-                                        logging.warning(T("Probable account sharing") + name)
-                                penalty = _PENALTY_SHARE
-                                block = True
-                            elif error.code in (452, 481, 482, 381) or (
-                                error.code in (500, 502) and clues_login(error.msg)
-                            ):
-                                # Cannot login, block this server
-                                if server.active:
-                                    errormsg = T("Failed login for server %s") % display_msg
-                                    if server.errormsg != errormsg:
-                                        server.errormsg = errormsg
-                                        logging.error(T("Failed login for server %s"), server.host)
-                                penalty = _PENALTY_PERM
-                                block = True
-                            elif error.code in (502, 482):
-                                # Cannot connect (other reasons), block this server
-                                if server.active:
-                                    errormsg = T("Cannot connect to server %s [%s]") % ("", error.msg)
-                                    if server.errormsg != errormsg:
-                                        server.errormsg = errormsg
-                                        logging.warning(T("Cannot connect to server %s [%s]"), server.host, error.msg)
-                                if clues_pay(error.msg):
-                                    penalty = _PENALTY_PERM
-                                else:
-                                    penalty = _PENALTY_502
-                                block = True
-                            elif error.code == 400:
-                                # Temp connection problem?
-                                if server.active:
-                                    logging.debug("Unspecified error 400 from server %s", server.host)
-                                penalty = _PENALTY_VERYSHORT
-                                block = True
-                            else:
-                                # Unknown error, just keep trying
-                                if server.active:
-                                    errormsg = T("Cannot connect to server %s [%s]") % ("", display_msg)
-                                    if server.errormsg != errormsg:
-                                        server.errormsg = errormsg
-                                        logging.warning(T("Cannot connect to server %s [%s]"), server.host, error.msg)
-                                penalty = _PENALTY_UNKNOWN
-                                block = True
-                            if block or (penalty and server.optional):
-                                retry_article = False
-                                if server.active:
-                                    if server.required:
-                                        sabnzbd.Scheduler.plan_required_server_resume()
-                                        retry_article = True
-                                    else:
-                                        server.deactivate()
-                                        if penalty and (block or server.optional):
-                                            self.plan_server(server, penalty)
-                                # Note that the article is discard for this server if the server is not required
-                                self.__reset_nw(nw, retry_article=retry_article, send_quit=True)
+                        if not self.__finish_connect_nw(nw):
                             continue
-                        except:
-                            logging.error(
-                                T("Connecting %s@%s failed, message=%s"),
-                                nw.thrdnum,
-                                nw.server.host,
-                                nntp_to_msg(nw.data),
-                            )
-                            # No reset-warning needed, above logging is sufficient
-                            self.__reset_nw(nw, retry_article=False)
-
                         if nw.connected:
                             logging.info("Connecting %s@%s finished", nw.thrdnum, nw.server.host)
                             self.__request_article(nw)
@@ -930,9 +812,13 @@ class Downloader(Thread):
                     # Successful data, clear "bad" counter
                     server.bad_cons = 0
                     server.errormsg = server.warning = ""
+
+                    # Update statistics and decode
+                    article.nzf.nzo.update_download_stats(BPSMeter.bps, server.id, nw.data_size)
+                    self.decode(article, nw.data, nw.data_size)
+
                     if sabnzbd.LOG_ALL:
                         logging.debug("Thread %s@%s: %s done", nw.thrdnum, server.host, article.article)
-                    self.decode(article, nw.data)
 
                     # Reset connection for new activity
                     nw.soft_reset()
@@ -940,6 +826,87 @@ class Downloader(Thread):
                     server.idle_threads.append(nw)
                     self.remove_socket(nw)
 
+    def __finish_connect_nw(self, nw: NewsWrapper) -> bool:
+        server = nw.server
+        try:
+            nw.finish_connect(nw.status_code)
+            if sabnzbd.LOG_ALL:
+                logging.debug("%s@%s last message -> %s", nw.thrdnum, server.host, nntp_to_msg(nw.data))
+            nw.clear_data()
+        except NNTPPermanentError as error:
+            # Handle login problems
+            block = False
+            penalty = 0
+            errormsg = None
+            logging.debug("Server login problem: %s", error.msg)
+            if error.code in (502, 400, 481, 482) and clues_too_many(error.msg):
+                # Too many connections: remove this thread and reduce thread-setting for server
+                # Plan to go back to the full number after a penalty timeout
+                errormsg = T("Too many connections to server %s [%s]") % (server.host, error.msg)
+                if server.active:
+                    # Don't count this for the tries (max_art_tries) on this server
+                    self.__reset_nw(nw, send_quit=True)
+                    self.plan_server(server, _PENALTY_TOOMANY)
+                    server.threads -= 1
+            elif error.code in (502, 481, 482) and clues_too_many_ip(error.msg):
+                # Login from (too many) different IP addresses
+                errormsg = T(
+                    "Login from too many different IP addresses to server %s [%s] - https://sabnzbd.org/multiple-adresses"
+                ) % (server.host, error.msg)
+                penalty = _PENALTY_SHARE
+                block = True
+            elif error.code in (452, 481, 482, 381) or (error.code in (500, 502) and clues_login(error.msg)):
+                # Cannot login, block this server
+                errormsg = T("Failed login for server %s [%s]") % (server.host, error.msg)
+                penalty = _PENALTY_PERM
+                block = True
+            elif error.code in (502, 482):
+                # Cannot connect (other reasons), block this server
+                errormsg = T("Cannot connect to server %s [%s]") % (server.host, error.msg)
+                if clues_pay(error.msg):
+                    penalty = _PENALTY_PERM
+                else:
+                    penalty = _PENALTY_502
+                block = True
+            elif error.code == 400:
+                # Temp connection problem?
+                logging.debug("Unspecified error 400 from server %s", server.host)
+                penalty = _PENALTY_VERYSHORT
+                block = True
+            else:
+                # Unknown error, just keep trying
+                errormsg = T("Cannot connect to server %s [%s]") % (server.host, error.msg)
+                penalty = _PENALTY_UNKNOWN
+                block = True
+            if block or (penalty and server.optional):
+                retry_article = False
+                if server.active:
+                    if server.required:
+                        sabnzbd.Scheduler.plan_required_server_resume()
+                        retry_article = True
+                    else:
+                        server.deactivate()
+                        if penalty and (block or server.optional):
+                            self.plan_server(server, penalty)
+                # Note that the article is discard for this server if the server is not required
+                self.__reset_nw(nw, retry_article=retry_article, send_quit=True)
+
+            # Set error for server and warn user if it was first time thrown
+            if errormsg and server.active and server.errormsg != errormsg:
+                server.errormsg = errormsg
+                logging.warning(errormsg)
+            return False
+        except:
+            logging.error(
+                T("Connecting %s@%s failed, message=%s"),
+                nw.thrdnum,
+                nw.server.host,
+                nntp_to_msg(nw.data),
+            )
+            # No reset-warning needed, above logging is sufficient
+            self.__reset_nw(nw, retry_article=False)
+        return True
+
     def __reset_nw(
         self,
         nw: NewsWrapper,
@@ -972,9 +939,9 @@ class Downloader(Thread):
                 nw.article.tries += 1
 
             # Do we discard, or try again for this server
-            if not retry_article or nw.article.tries > cfg.max_art_tries():
+            if not retry_article or (not nw.server.required and nw.article.tries > cfg.max_art_tries()):
                 # Too many tries on this server, consider article missing
-                self.decode(nw.article, None)
+                self.decode(nw.article)
                 nw.article.tries = 0
             else:
                 # Retry again with the same server
