@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -72,11 +72,11 @@ RE_IP4 = re.compile(r"inet\s+(addr:\s*)?(\d+\.\d+\.\d+\.\d+)")
 RE_IP6 = re.compile(r"inet6\s+(addr:\s*)?([0-9a-f:]+)", re.I)
 
 # Check if strings are defined for AM and PM
-HAVE_AMPM = bool(time.strftime("%p", time.localtime()))
+HAVE_AMPM = bool(time.strftime("%p"))
 
 
 def helpful_warning(*args, **kwargs):
-    """Wrapper to ignore helpfull warnings if desired"""
+    """Wrapper to ignore helpful warnings if desired"""
     if sabnzbd.cfg.helpful_warnings():
         return logging.warning(*args, **kwargs)
     return logging.info(*args, **kwargs)
@@ -161,7 +161,7 @@ def safe_lower(txt: Any) -> str:
 
 def cmp(x, y):
     """
-    Replacement for built-in funciton cmp that was removed in Python 3
+    Replacement for built-in function cmp that was removed in Python 3
 
     Compare the two objects x and y and return an integer according to
     the outcome. The return value is negative if x < y, zero if x == y
@@ -231,14 +231,13 @@ def pp_to_opts(pp: int) -> Tuple[bool, bool, bool]:
 
 def opts_to_pp(repair: bool, unpack: bool, delete: bool) -> int:
     """Convert (repair, unpack, delete) to numeric process options"""
-    pp = 0
-    if repair:
-        pp = 1
-    if unpack:
-        pp = 2
     if delete:
-        pp = 3
-    return pp
+        return 3
+    if unpack:
+        return 2
+    if repair:
+        return 1
+    return 0
 
 
 _wildcard_to_regex = {
@@ -408,7 +407,7 @@ def check_latest_version():
     Formula for the version numbers (line 1 and 3).
         <major>.<minor>.<bugfix>[rc|beta|alpha]<cand>
 
-    The <cand> value for a final version is assumned to be 99.
+    The <cand> value for a final version is assumed to be 99.
     The <cand> value for the beta/rc version is 1..98, with RC getting
     a boost of 80 and Beta of 40.
     This is done to signal alpha/beta/rc users of availability of the final
@@ -504,7 +503,8 @@ def upload_file_to_sabnzbd(url, fp):
 
 
 def from_units(val: str) -> float:
-    """Convert K/M/G/T/P notation to float"""
+    """Convert K/M/G/T/P notation to float
+    Does not support negative numbers"""
     val = str(val).strip().upper()
     if val == "-1":
         return float(val)
@@ -530,30 +530,29 @@ def from_units(val: str) -> float:
 def to_units(val: Union[int, float], postfix="") -> str:
     """Convert number to K/M/G/T/P notation
     Show single decimal for M and higher
+    Also supports negative numbers
     """
-    dec_limit = 1
+    if not isinstance(val, (int, float)):
+        return ""
+
     if val < 0:
         sign = "-"
     else:
         sign = ""
-    val = str(abs(val)).strip()
 
+    # Determine what form we are at
+    val = abs(val)
     n = 0
-    try:
-        val = float(val)
-    except:
-        return ""
-    while (val > 1023.0) and (n < 5):
-        val = val / 1024.0
+    while (val > 1023) and (n < 5):
+        val = val / 1024
         n = n + 1
-    unit = TAB_UNITS[n]
-    if n > dec_limit:
+
+    if n > 1:
         decimals = 1
     else:
         decimals = 0
 
-    fmt = "%%s%%.%sf %%s%%s" % decimals
-    return fmt % (sign, val, unit, postfix)
+    return ("%%s%%.%sf %%s%%s" % decimals) % (sign, val, TAB_UNITS[n], postfix)
 
 
 def caller_name(skip=2):
@@ -583,8 +582,13 @@ def caller_name(skip=2):
 
 def exit_sab(value: int):
     """Leave the program after flushing stderr/stdout"""
-    sys.stderr.flush()
-    sys.stdout.flush()
+    try:
+        sys.stderr.flush()
+        sys.stdout.flush()
+    except AttributeError:
+        # Not supported on Windows binaries
+        pass
+
     # Cannot use sys.exit as it will not work inside the macOS-runner-thread
     os._exit(value)
 
@@ -721,14 +725,12 @@ def loadavg():
     """Return 1, 5 and 15 minute load average of host or "" if not supported"""
     p = ""
     if not sabnzbd.WIN32 and not sabnzbd.MACOS:
-        opt = cfg.show_sysload()
-        if opt:
-            try:
-                p = "%.2f | %.2f | %.2f" % os.getloadavg()
-            except:
-                pass
-            if opt > 1 and _HAVE_STATM:
-                p = "%s | %s" % (p, memory_usage())
+        try:
+            p = "%.2f | %.2f | %.2f" % os.getloadavg()
+        except:
+            pass
+        if _HAVE_STATM:
+            p = "%s | %s" % (p, memory_usage())
     return p
 
 
@@ -804,9 +806,10 @@ def get_all_passwords(nzo) -> List[str]:
         logging.info("Found a password that was set by the user: %s", nzo.password)
         passwords.append(nzo.password.strip())
 
+    # Note that we get a reference to the list, so adding to it updates the original list!
     meta_passwords = nzo.meta.get("password", [])
     pw = nzo.nzo_info.get("password")
-    if pw:
+    if pw and pw not in meta_passwords:
         meta_passwords.append(pw)
 
     if meta_passwords:
@@ -1105,7 +1108,7 @@ def list2cmdline_unrar(lst: List[str]) -> str:
 
 
 def build_and_run_command(command: List[str], windows_unrar_command: bool = False, **kwargs):
-    """Builds and then runs command with nessecary flags and optional
+    """Builds and then runs command with necessary flags and optional
     IONice and Nice commands. Optional Popen arguments can be supplied.
     On Windows we need to run our own list2cmdline for Unrar.
     Returns the Popen-instance.
@@ -1327,19 +1330,18 @@ def system_standby():
         sabnzbd.powersup.linux_standby()
 
 
-def change_queue_complete_action(action, new=True):
+def change_queue_complete_action(action: str, new: bool = True):
     """Action or script to be performed once the queue has been completed
     Scripts are prefixed with 'script_'
-    When "new" is False, check whether non-script actions are acceptable
     """
     _action = None
     _argument = None
-    if action.startswith("script_") and is_valid_script(action.replace("script_", "", 1)):
-        # all scripts are labeled script_xxx
-        _action = sabnzbd.misc.run_script
-        _argument = action.replace("script_", "", 1)
-    elif new or cfg.queue_complete_pers():
-        if action == "shutdown_pc":
+    if new or cfg.queue_complete_pers():
+        if action.startswith("script_") and is_valid_script(action.replace("script_", "", 1)):
+            # all scripts are labeled script_xxx
+            _action = sabnzbd.misc.run_script
+            _argument = action.replace("script_", "", 1)
+        elif action == "shutdown_pc":
             _action = system_shutdown
         elif action == "hibernate_pc":
             _action = system_hibernate
@@ -1367,7 +1369,7 @@ def keep_awake():
         if sabnzbd.cfg.keep_awake():
             ES_CONTINUOUS = 0x80000000
             ES_SYSTEM_REQUIRED = 0x00000001
-            if (not sabnzbd.Downloader.is_paused() and not sabnzbd.NzbQueue.is_empty()) or (
+            if (not sabnzbd.Downloader.no_active_jobs() and not sabnzbd.NzbQueue.is_empty()) or (
                 not sabnzbd.PostProcessor.paused and not sabnzbd.PostProcessor.empty()
             ):
                 if sabnzbd.KERNEL32:
