@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -56,6 +56,7 @@ class NewsWrapper:
         "timeout",
         "article",
         "data",
+        "data_size",
         "nntp",
         "connected",
         "user_sent",
@@ -75,6 +76,7 @@ class NewsWrapper:
         self.timeout: Optional[float] = None
         self.article: Optional[sabnzbd.nzbstuff.Article] = None
         self.data: List[AnyStr] = []
+        self.data_size: int = 0
 
         self.nntp: Optional[NNTP] = None
 
@@ -176,28 +178,18 @@ class NewsWrapper:
         self.nntp.sock.sendall(command)
         self.clear_data()
 
-    def recv_chunk(self, block: bool = False) -> Tuple[int, bool, bool]:
-        """Receive data, return #bytes, done, skip"""
-        self.timeout = time.time() + self.server.timeout
-        while 1:
-            try:
-                if self.nntp.nw.server.ssl:
-                    # SSL chunks come in 16K frames
-                    # Setting higher limits results in slowdown
-                    chunk = self.nntp.sock.recv(16384)
-                else:
-                    # Get as many bytes as possible
-                    chunk = self.nntp.sock.recv(262144)
-                break
-            except ssl.SSLWantReadError:
-                # SSL connections will block until they are ready.
-                # Either ignore the connection until it responds
-                # Or wait in a loop until it responds
-                if block:
-                    # time.sleep(0.0001)
-                    continue
-                else:
-                    return 0, False, True
+    def recv_chunk(self) -> Tuple[int, bool]:
+        """Receive data, return #bytes, done"""
+        if self.nntp.nw.server.ssl:
+            # SSL chunks come in 16K frames
+            # Setting higher limits results in slowdown
+            chunk = self.nntp.sock.recv(16384)
+        else:
+            chunk = self.nntp.sock.recv(262144)
+
+        chunk_len = len(chunk)
+        if chunk_len == 0:
+            raise ConnectionError("server closed connection")
 
         if not self.data:
             try:
@@ -207,20 +199,20 @@ class NewsWrapper:
 
         # Append so we can do 1 join(), much faster than multiple!
         self.data.append(chunk)
+        self.data_size += chunk_len
+        self.timeout = time.time() + self.server.timeout
 
         # Official end-of-article is ".\r\n" but sometimes it can get lost between 2 chunks
-        chunk_len = len(chunk)
         if chunk[-5:] == b"\r\n.\r\n":
-            return chunk_len, True, False
+            return chunk_len, True
         elif chunk_len < 5 and len(self.data) > 1:
             # We need to make sure the end is not split over 2 chunks
             # This is faster than join()
-            combine_chunk = self.data[-2][-5:] + chunk
-            if combine_chunk[-5:] == b"\r\n.\r\n":
-                return chunk_len, True, False
+            if self.data[-2][-5 + chunk_len :] + chunk == b"\r\n.\r\n":
+                return chunk_len, True
 
         # Still in middle of data, so continue!
-        return chunk_len, False, False
+        return chunk_len, False
 
     def soft_reset(self):
         """Reset for the next article"""
@@ -231,6 +223,7 @@ class NewsWrapper:
     def clear_data(self):
         """Clear the stored raw data"""
         self.data = []
+        self.data_size = 0
         self.status_code = None
 
     def hard_reset(self, wait: bool = True, send_quit: bool = True):
@@ -333,9 +326,8 @@ class NNTP:
     def connect(self):
         """Start of connection, can be performed a-sync"""
         try:
-            # Wait only 15 seconds during server test
-            if self.nw.blocking:
-                self.sock.settimeout(15)
+            # Wait the defined timeout during connect and SSL-setup
+            self.sock.settimeout(self.nw.server.timeout)
 
             # Connect
             self.sock.connect((self.host, self.nw.server.port))
@@ -353,12 +345,11 @@ class NNTP:
                 )
                 self.nw.server.ssl_info = "%s (%s)" % (self.sock.version(), self.sock.cipher()[0])
 
-            # Set blocking mode
-            self.sock.setblocking(self.nw.blocking)
-
-            # Now it's safe to add the socket to the list of active sockets
-            # Skip this step during server test
+            # Skip during server test
             if not self.nw.blocking:
+                # Set to non-blocking mode
+                self.sock.setblocking(False)
+                # Now it's safe to add the socket to the list of active sockets
                 sabnzbd.Downloader.add_socket(self.fileno, self.nw)
         except OSError as e:
             self.error(e)
@@ -399,10 +390,13 @@ class NNTP:
             raise socket.error(errno.ECONNREFUSED, str(error))
         else:
             msg = "Failed to connect: %s" % (str(error))
-            msg = "%s %s@%s:%s" % (msg, self.nw.thrdnum, self.host, self.nw.server.port)
+            msg = "%s %s:%s (%s)" % (msg, self.nw.server.host, self.nw.server.port, self.host)
             self.error_msg = msg
             self.nw.server.next_busy_threads_check = 0
-            logging.info(msg)
+            if self.nw.server.warning == msg:
+                logging.info(msg)
+            else:
+                logging.warning(msg)
             self.nw.server.warning = msg
 
     def __repr__(self):
