@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,7 +23,8 @@ sabnzbd.notifier - Send notifications to any notification services
 
 import os.path
 import logging
-import urllib.request, urllib.error, urllib.parse
+import urllib.request
+import urllib.parse
 import http.client
 import json
 from threading import Thread
@@ -121,7 +122,7 @@ def send_notification(title, msg, gtype, job_cat=None):
     """Send Notification message"""
     logging.info("Sending notification: %s - %s (type=%s, job_cat=%s)", title, msg, gtype, job_cat)
     # Notification Center
-    if sabnzbd.DARWIN and sabnzbd.cfg.ncenter_enable():
+    if sabnzbd.MACOS and sabnzbd.cfg.ncenter_enable():
         if check_classes(gtype, "ncenter") and check_cat("ncenter", job_cat):
             send_notification_center(title, msg, gtype)
 
