@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -112,11 +112,19 @@ class TryList:
     def __init__(self):
         self.try_list: List[Server] = []
 
-    def server_in_try_list(self, server: Server):
+    def server_in_try_list(self, server: Server) -> bool:
         """Return whether specified server has been tried"""
         with TRYLIST_LOCK:
             return server in self.try_list
 
+    def all_servers_in_try_list(self, servers: List[Server]) -> bool:
+        """Check if all servers have been tried"""
+        with TRYLIST_LOCK:
+            for server in servers:
+                if not server in self.try_list:
+                    return False
+        return True
+
     def add_to_try_list(self, server: Server):
         """Register server as having been tried already"""
         with TRYLIST_LOCK:
@@ -178,66 +186,26 @@ class Article(TryList):
 
     def get_article(self, server: Server, servers: List[Server]):
         """Return article when appropriate for specified server"""
-        log = sabnzbd.LOG_ALL
-        if not self.fetcher and not self.server_in_try_list(server):
-            if log:
-                logging.debug("Article %s | Server: %s | in second if", self.article, server.host)
-                # Is the current selected server of the same priority as this article?
-                logging.debug(
-                    "Article %s | Server: %s | Article priority: %s", self.article, server.host, self.fetcher_priority
-                )
-                logging.debug(
-                    "Article %s | Server: %s | Server priority: %s", self.article, server.host, server.priority
-                )
-            if server.priority == self.fetcher_priority:
-                self.fetcher = server
-                self.tries += 1
-                if log:
-                    logging.debug("Article %s | Server: %s | same priority, use it", self.article, server.host)
-                    logging.debug("Article %s | Server: %s | Article-try: %s", self.article, server.host, self.tries)
-                return self
-            else:
-                if log:
-                    logging.debug("Article %s | Server: %s | not the same priority", self.article, server.host)
-                # No, so is it a lower priority?
-                if server.priority > self.fetcher_priority:
-                    if log:
-                        logging.debug("Article %s | Server: %s | lower priority", self.article, server.host)
-                    # Is there an available server that is a higher priority?
-                    found_priority = 1000
-                    # for server_check in config.get_servers():
-                    for server_check in servers:
-                        if log:
-                            logging.debug("Article %s | Server: %s | checking", self.article, server.host)
-                        if server_check.active and (server_check.priority < found_priority):
-                            if server_check.priority < server.priority:
-                                if not self.server_in_try_list(server_check):
-                                    if log:
-                                        logging.debug(
-                                            "Article %s | Server: %s | setting found priority to %s",
-                                            self.article,
-                                            server.host,
-                                            server_check.priority,
-                                        )
-                                    found_priority = server_check.priority
-                    if found_priority == 1000:
-                        # If no higher priority servers, use this server
-                        self.fetcher_priority = server.priority
-                        self.fetcher = server
-                        self.tries += 1
-                        if log:
-                            logging.debug(
-                                "Article %s | Server: %s | Article-try: %s", self.article, server.host, self.tries
-                            )
-                        return self
-                    else:
-                        # There is a higher priority server, so set article priority
-                        if log:
-                            logging.debug("Article %s | Server: %s | setting self priority", self.article, server.host)
-                        self.fetcher_priority = found_priority
-        if log:
-            logging.debug("Article %s | Server: %s | Returning None", self.article, server.host)
-        return None
+        if self.fetcher or self.server_in_try_list(server):
+            return None
+
+        if server.priority > self.fetcher_priority:
+            # Check for higher priority server, taking advantage of servers list being sorted by priority
+            for server_check in servers:
+                if server_check.priority < server.priority:
+                    if server_check.active and not self.server_in_try_list(server_check):
+                        # There is a higher priority server, so set article priority and return
+                        self.fetcher_priority = server_check.priority
+                        return None
+                else:
+                    # All servers with a higher priority have been checked
+                    break
+
+        # If no higher priority servers, use this server
+        self.fetcher_priority = server.priority
+        self.fetcher = server
+        self.tries += 1
+        return self
 
     def get_art_id(self):
         """Return unique article storage name, create if needed"""
@@ -290,7 +258,7 @@ class Article(TryList):
     def __hash__(self):
         """Required because we implement eq. Articles with the same
         usenet address can appear in different NZF's. So we make every
-        article object unique, even though it is bad pratice.
+        article object unique, even though it is bad practice.
         """
         return id(self)
 
@@ -311,7 +279,6 @@ NzbFileSaver = (
     "vol",
     "blocks",
     "setname",
-    "has_bad_articles",
     "articles",
     "decodetable",
     "bytes",
@@ -348,8 +315,6 @@ class NzbFile(TryList):
         self.blocks: Optional[int] = None
         self.setname: Optional[str] = None
 
-        self.has_bad_articles: bool = False
-
         # Articles are removed from "articles" after being fetched
         self.articles: List[Article] = []
         self.decodetable: List[Article] = []
@@ -494,12 +459,12 @@ class NzbFile(TryList):
         # Set non-transferable values
         self.md5 = None
 
-    def __eq__(self, other):
-        """Assume it's the same file if the numer bytes and first article
+    def __eq__(self, other: "NzbFile"):
+        """Assume it's the same file if the number bytes and first article
         are the same or if there are no articles left, use the filenames.
         Some NZB's are just a mess and report different sizes for the same article.
         """
-        if self.bytes == other.bytes or len(self.decodetable) == len(other.decodetable):
+        if other and (self.bytes == other.bytes or len(self.decodetable) == len(other.decodetable)):
             if self.decodetable and other.decodetable:
                 return self.decodetable[0] == other.decodetable[0]
             # Fallback to filename comparison
@@ -509,7 +474,7 @@ class NzbFile(TryList):
     def __hash__(self):
         """Required because we implement eq. The same file can be spread
         over multiple NZO's so we make every NZF unique. Even though
-        it's considered bad pratice.
+        it's considered bad practice.
         """
         return id(self)
 
@@ -566,7 +531,6 @@ NzbObjectSaver = (
     "nzo_id",
     "futuretype",
     "removed_from_queue",
-    "parsed",
     "action_line",
     "unpack_info",
     "fail_msg",
@@ -709,7 +673,6 @@ class NzbObject(TryList):
         self.futuretype = futuretype
         self.removed_from_queue = False
         self.to_be_removed = False
-        self.parsed = False
         self.duplicate = False
         self.oversized = False
         self.precheck = False
@@ -761,6 +724,9 @@ class NzbObject(TryList):
         if cfg.replace_spaces():
             logging.info("Replacing spaces with underscores in %s", self.final_name)
             self.final_name = self.final_name.replace(" ", "_")
+        if cfg.replace_underscores():
+            logging.info("Replacing underscores with dots in %s", self.final_name)
+            self.final_name = self.final_name.replace("_", ".")
         if cfg.replace_dots():
             logging.info("Replacing dots with spaces in %s", self.final_name)
             self.final_name = self.final_name.replace(".", " ")
@@ -840,6 +806,10 @@ class NzbObject(TryList):
         self.set_priority(priority)
         self.repair, self.unpack, self.delete = pp_to_opts(pp_tmp)
 
+        # Show first meta-password (if any), when there's no explicit password
+        if not self.password and self.meta.get("password"):
+            self.password = self.meta.get("password", [None])[0]
+
         # Run user pre-queue script if set and valid
         if not reuse and make_script_path(cfg.pre_script()):
             # Call the script
@@ -967,10 +937,6 @@ class NzbObject(TryList):
                 self.nzo_info[kw] = self.meta[kw][0]
         logging.debug("NZB nzo-info = %s", self.nzo_info)
 
-        # Show first meta-password (if any), when there's no explicit password
-        if not self.password and self.meta.get("password"):
-            self.password = self.meta.get("password", [None])[0]
-
         # Set nzo save-delay to minimum 120 seconds
         self.save_timeout = max(120, min(6.0 * self.bytes / GIGI, 300.0))
 
