@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,10 +19,8 @@
 sabnzbd.osxmenu - macOS Top Menu
 """
 
-import objc
 from Foundation import *
 from AppKit import *
-from PyObjCTools import AppHelper
 from objc import YES, NO
 
 import os
@@ -40,7 +38,6 @@ from sabnzbd.panic import launch_a_browser
 
 from sabnzbd.api import fast_queue
 import sabnzbd.config as config
-import sabnzbd.downloader
 
 status_icons = {
     "idle": "icons/sabnzbd_osx_idle.tiff",
@@ -51,14 +48,13 @@ start_time = NSDate.date()
 
 
 class SABnzbdDelegate(NSObject):
-
     icons = {}
     status_bar = None
     history_db = None
 
     def awakeFromNib(self):
         # Do we want the menu
-        if sabnzbd.cfg.osx_menu():
+        if sabnzbd.cfg.tray_icon():
             # Status Bar initialize
             self.buildMenu()
 
@@ -80,9 +76,7 @@ class SABnzbdDelegate(NSObject):
                 # Path is modified for the binary
                 icon_path = os.path.join(os.path.dirname(sys.executable), "..", "Resources", status_icons[icon])
             self.icons[icon] = NSImage.alloc().initByReferencingFile_(icon_path)
-            if sabnzbd.DARWIN_VERSION > 9:
-                # Support for Yosemite Dark Mode
-                self.icons[icon].setTemplate_(YES)
+            self.icons[icon].setTemplate_(YES)
 
         self.status_item.setImage_(self.icons["idle"])
         self.status_item.setAlternateImage_(self.icons["clicked"])
@@ -97,8 +91,6 @@ class SABnzbdDelegate(NSObject):
         # Variables
         self.state = "Idle"
         self.speed = 0
-        self.version_notify = 1
-        self.status_removed = 0
 
         # Menu construction
         self.menu = NSMenu.alloc().init()
@@ -117,6 +109,7 @@ class SABnzbdDelegate(NSObject):
         self.menu.addItem_(self.state_menu_item)
 
         # Queue Item
+        self.menu_queue = None
         self.queue_menu_item = NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(
             T("Queue"), "openBrowserAction:", ""
         )
@@ -132,6 +125,7 @@ class SABnzbdDelegate(NSObject):
         self.menu.addItem_(self.purgequeue_menu_item)
 
         # History Item
+        self.menu_history = None
         self.history_menu_item = NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(
             T("History"), "openBrowserAction:", ""
         )
@@ -250,31 +244,33 @@ class SABnzbdDelegate(NSObject):
 
     def queueUpdate(self):
         try:
-            qnfo = sabnzbd.NzbQueue.queue_info(start=0, limit=10)
+            queue_bytes_total, queue_bytes_left, _, nzo_list, _, queue_fullsize = sabnzbd.NzbQueue.queue_info(limit=10)
             bytesleftprogess = 0
             self.info = ""
-            self.menu_queue = NSMenu.alloc().init()
 
-            if qnfo.list:
+            if not self.menu_queue:
+                self.menu_queue = NSMenu.alloc().init()
+            self.menu_queue.removeAllItems()
+
+            if nzo_list:
                 menu_queue_item = NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(
                     T("Queue First 10 Items"), "", ""
                 )
                 self.menu_queue.addItem_(menu_queue_item)
                 self.menu_queue.addItem_(NSMenuItem.separatorItem())
 
-                for pnfo in qnfo.list:
-                    bytesleft = pnfo.bytes_left / MEBI
-                    bytesleftprogess += pnfo.bytes_left
-                    bytes_total = pnfo.bytes / MEBI
+                for nzo in nzo_list:
+                    bytesleft = nzo.remaining / MEBI
+                    bytesleftprogess += nzo.remaining
+                    bytes_total = nzo.bytes / MEBI
                     timeleft = sabnzbd.api.calc_timeleft(bytesleftprogess, sabnzbd.BPSMeter.bps)
-                    job = "%s\t(%d/%d MB) %s" % (pnfo.filename, bytesleft, bytes_total, timeleft)
-                    menu_queue_item = NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(job, "", "")
-                    self.menu_queue.addItem_(menu_queue_item)
+                    job = "%s\t(%d/%d MB) %s" % (nzo.filename, bytesleft, bytes_total, timeleft)
+                    self.menu_queue.addItem_(NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(job, "", ""))
 
                 self.info = "%d nzb(s)\t(%d / %d MB)" % (
-                    qnfo.q_size_list,
-                    (qnfo.bytes_left / MEBI),
-                    (qnfo.bytes / MEBI),
+                    queue_fullsize,
+                    (queue_bytes_left / MEBI),
+                    (queue_bytes_total / MEBI),
                 )
             else:
                 menu_queue_item = NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(T("Empty"), "", "")
@@ -290,11 +286,10 @@ class SABnzbdDelegate(NSObject):
                 self.history_db = sabnzbd.database.HistoryDB()
             items = self.history_db.fetch_history(limit=10)[0]
 
-            self.menu_history = NSMenu.alloc().init()
-            self.failedAttributes = {
-                NSForegroundColorAttributeName: NSColor.redColor(),
-                NSFontAttributeName: NSFont.menuFontOfSize_(14.0),
-            }
+            if not self.menu_history:
+                self.menu_history = NSMenu.alloc().init()
+            self.menu_history.removeAllItems()
+
             menu_history_item = NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(
                 T("History Last 10 Items"), "", ""
             )
@@ -313,7 +308,11 @@ class SABnzbdDelegate(NSObject):
                         )
                     if history["status"] != Status.COMPLETED:
                         jobfailed = NSAttributedString.alloc().initWithString_attributes_(
-                            history["name"], self.failedAttributes
+                            history["name"],
+                            {
+                                NSForegroundColorAttributeName: NSColor.redColor(),
+                                NSFontAttributeName: NSFont.menuFontOfSize_(14.0),
+                            },
                         )
                         menu_history_item.setAttributedTitle_(jobfailed)
                     menu_history_item.setRepresentedObject_("%s" % history["storage"])
@@ -361,16 +360,7 @@ class SABnzbdDelegate(NSObject):
                 if "M" in speed and len(speed) > 5:
                     speed = speed.replace(" ", "")
                 time_left = (bpsnow > 10 and time_left) or "------"
-
-                statusbarText = "\n\n%s\n%sB/s\n" % (time_left, speed)
-
-                if sabnzbd.SABSTOP:
-                    statusbarText = "..."
-
-                if not sabnzbd.cfg.osx_speed():
-                    statusbarText = ""
-
-                self.setMenuTitle_(statusbarText)
+                self.setMenuTitle_("\n\n%s\n%sB/s\n" % (time_left, speed))
             else:
                 self.state = T("Idle")
                 self.setMenuTitle_("")
@@ -402,13 +392,12 @@ class SABnzbdDelegate(NSObject):
 
     def speedlimitUpdate(self):
         try:
-            speed = int(sabnzbd.Downloader.get_limit())
-            if self.speed != speed:
-                self.speed = speed
+            if self.speed != sabnzbd.Downloader.bandwidth_perc:
+                self.speed = sabnzbd.Downloader.bandwidth_perc
                 speedsValues = self.menu_speed.numberOfItems()
                 for i in range(speedsValues):
                     menuitem = self.menu_speed.itemAtIndex_(i)
-                    if speed == int(menuitem.representedObject()):
+                    if sabnzbd.Downloader.bandwidth_perc == int(menuitem.representedObject()):
                         menuitem.setState_(NSOnState)
                     else:
                         menuitem.setState_(NSOffState)
@@ -435,13 +424,8 @@ class SABnzbdDelegate(NSObject):
             style.setMaximumLineHeight_(9.0)
             style.setParagraphSpacing_(-3.0)
 
-            # In Big Sur the offset was changed
-            baseline_offset = 5.0
-            if sabnzbd.DARWIN_VERSION >= 16:
-                baseline_offset = baseline_offset * -1
-
             titleAttributes = {
-                NSBaselineOffsetAttributeName: baseline_offset,
+                NSBaselineOffsetAttributeName: -5.0,
                 NSFontAttributeName: NSFont.menuFontOfSize_(9.0),
                 NSParagraphStyleAttributeName: style,
             }
@@ -522,7 +506,7 @@ class SABnzbdDelegate(NSObject):
             logging.info("[osx] receiving from macOS : %s", filename)
             if os.path.exists(filename):
                 if sabnzbd.filesystem.get_ext(filename) in VALID_ARCHIVES + VALID_NZB_FILES:
-                    sabnzbd.add_nzbfile(filename, keep=True)
+                    sabnzbd.nzbparser.add_nzbfile(filename, keep=True)
         # logging.info('opening done')
 
     def applicationShouldTerminate_(self, sender):
