@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -352,7 +352,6 @@ class SeriesSorter(BaseSorter):
         guess: Optional[MatchesDict] = None,
         force: Optional[bool] = False,
     ):
-
         super().__init__(nzo, job_name, path, cat, cfg.tv_sort_string(), cfg.tv_categories(), guess, force)
 
     def match(self):
@@ -506,7 +505,6 @@ class DateSorter(BaseSorter):
         guess: Optional[MatchesDict] = None,
         force: Optional[bool] = False,
     ):
-
         super().__init__(nzo, job_name, path, cat, cfg.date_sort_string(), cfg.date_categories(), guess, force)
 
     def match(self):
@@ -601,6 +599,7 @@ def guess_what(name: str, sort_type: Optional[str] = None) -> MatchesDict:
         # "allowed_countries": [],
         # "allowed_languages": [],
         "excludes": EXCLUDED_GUESSIT_PROPERTIES,
+        "date_year_first": True,  # Make sure also short-dates are detected as YY-MM-DD
     }
     if sort_type:
         # Hint the type if known
@@ -737,14 +736,6 @@ def get_descriptions(nzo: Optional[NzbObject], guess: Optional[MatchesDict], job
     return ep_name.replace("_", " "), ep_name.replace(" - ", "-").replace(" ", "."), ep_name.replace(" ", "_")
 
 
-def has_subdirectory(path: str) -> bool:
-    """Return True if any directory is found inside the tree at 'path'"""
-    for _root, dirs, _files in os.walk(path):
-        if dirs:
-            return True
-    return False
-
-
 def to_lowercase(path: str) -> str:
     """Lowercases any characters enclosed in {}"""
     RE_LOWERCASE = re.compile(r"{([^{]*)}")
