@@ -1,6 +1,8 @@
 import platform
 import subprocess
 import locale
+import logging
+import time
 from .pystone import pystones
 
 
@@ -38,15 +40,21 @@ def getcpu():
         # OK, found. Remove unwanted spaces:
         cputype = " ".join(cputype.split())
     else:
-        # Not found, so let's fall back to platform()
-        cputype = platform.platform()
+        try:
+            # Not found, so let's fall back to platform()
+            cputype = platform.platform()
+        except:
+            # Can fail on special platforms (like Snapcraft or embedded)
+            pass
 
+    logging.debug("CPU model = %s", cputype)
     return cputype
 
 
 def getpystone():
     # Start calculation
     maxpystone = 0
+    start = time.time()
     # Start with a short run, find the the pystone, and increase runtime until duration took > 0.1 second
     for pyseed in [1000, 2000, 5000, 10000, 20000, 50000, 100000, 200000]:
         duration, pystonefloat = pystones(pyseed)
@@ -54,6 +62,8 @@ def getpystone():
         # Stop when pystone() has been running for at least 0.1 second
         if duration > 0.1:
             break
+
+    logging.debug("Pystone performance = %d (in %.2f seconds)", maxpystone, time.time() - start)
     return maxpystone
 
 
